#!/usr/bin/perl -w
# NAME: csv2html.pl
# AIM:
use strict;
use warnings;
use File::Basename;  # split path ($name,$dir,$ext) = fileparse($file [, qr/\.[^.]*/] )
use Cwd;
my $os = $^O;
my $perl_dir = '/home/geoff/bin';
my $PATH_SEP = '/';
my $temp_dir = '/tmp';
if ($os =~ /win/i) {
    $perl_dir = 'C:\GTools\perl';
    $temp_dir = $perl_dir;
    $PATH_SEP = "\\";
}
unshift(@INC, $perl_dir);
require 'lib_utils.pl' or die "Unable to load 'lib_utils.pl' Check paths in \@INC...\n";
# log file stuff
our ($LF);
my $pgmname = $0;
if ($pgmname =~ /(\\|\/)/) {
    my @tmpsp = split(/(\\|\/)/,$pgmname);
    $pgmname = $tmpsp[-1];
}
my $outfile = $temp_dir.$PATH_SEP."temp.$pgmname.txt";
open_log($outfile);

# user variables
my $VERS = "0.0.5 2015-01-09";
my $load_log = 0;
my $in_file = 'F:\FGx\fgx-aircraft\aircraft-status-01.csv';
my $verbosity = 0;
my $out_file = 'F:\FGx\fgx-aircraft\aircraft.html';

# ### DEBUG ###
my $debug_on = 0;
my $def_file = 'def_file';

### program variables
my @warnings = ();
my $cwd = cwd();

sub VERB1() { return $verbosity >= 1; }
sub VERB2() { return $verbosity >= 2; }
sub VERB5() { return $verbosity >= 5; }
sub VERB9() { return $verbosity >= 9; }

sub show_warnings($) {
    my ($val) = @_;
    if (@warnings) {
        prt( "\nGot ".scalar @warnings." WARNINGS...\n" );
        foreach my $itm (@warnings) {
           prt("$itm\n");
        }
        prt("\n");
    } else {
        prt( "\nNo warnings issued.\n\n" ) if (VERB9());
    }
}

sub pgm_exit($$) {
    my ($val,$msg) = @_;
    if (length($msg)) {
        $msg .= "\n" if (!($msg =~ /\n$/));
        prt($msg);
    }
    show_warnings($val);
    close_log($outfile,$load_log);
    exit($val);
}


sub prtw($) {
   my ($tx) = shift;
   $tx =~ s/\n$//;
   prt("$tx\n");
   push(@warnings,$tx);
}

sub html_head() {
    my $txt = <<EOF;
<!DOCTYPE html>
<html>
<head>
<title>Aircraft</title>
<style>
table { align: center; }
.ctr { text-align: center; }
p.top { 
margin : 0;
border-style : none;
padding : 0;
text-align : center;
}
</style>
</head>
<body>
<a name="top"></a>
<h1>Aircraft</h1>
<div>
<table border="1">
EOF
    return $txt;
}

sub html_tail() {
    my $txt = <<EOF;
</table>
</div>
<p class="top"><a href="#top">top</a></p>
</body>
</html>
EOF
    return $txt;
}

# 0        1     2         3          4
# aircraft,usage,json-file,direction, dims actual,bounding box,scale,Wikipedia,missing-bits,extra-bits

sub process_in_file($) {
    my ($inf) = @_;
    if (! open INF, "<$inf") {
        pgm_exit(1,"ERROR: Unable to open file [$inf]\n"); 
    }
    my @lines = <INF>;
    close INF;
    my $lncnt = scalar @lines;
    my $loader = "../fgx-aircraft-overview/r8/aircraft-overview.html";
    prt("Processing $lncnt lines, from [$inf]...\n");
    my ($line,$inc,$lnn,@arr,$len,$cnt,$i,$link);
    $lnn = 0;
    my $html = '';
    my $wrap = 30;
    my $row = 0;
    foreach $line (@lines) {
        chomp $line;
        $lnn++;
        $line = trim_all($line);
        $len = length($line);
        @arr = split(',',$line);
        $cnt = scalar @arr;
        if ($cnt > 4) {
            $html .= " <tr>\n";
            if ($lnn == 1) {
                for ($i = 0; $i < 5; $i++) {
                    $html .= "  <th>\n";
                    if ($i == 4) {
                        $html .= "   done\n";
                    } else {
                        $html .= "   ".$arr[$i]."\n";
                    }
                    $html .= "  </th>\n";
                }
            } else {
                for ($i = 0; $i < 5; $i++) {
                    $html .= "  <td>\n";
                    $inc = $arr[$i];
                    $len = length($inc);
                    if ($len == 0) {
                        $inc = "&nbsp;";
                    } elsif ($i == 0) {
                        $inc = "<a target=\"_blank\" href=\"".$loader."?model=$inc\">$inc</a>";
                    }
                    $html .= "   $inc\n";
                    $html .= "  </td>\n";
                }
                $row++;
            }
            $html .= " </tr>\n";
            if ($row == $wrap) {
                $html .= "<tr><td colspan=\"5\">\n";
                $html .= "<p class=\"top\"><a href=\"#top\">top</a></p>\n";
                $html .= "</td></tr>\n";
                $row = 0;
            }
        }
    }
    $html .= "<tr><td colspan=\"5\"><p class=\"top\"><a href=\"#top\">top</a></p></td></tr>\n";
    $html = html_head().$html.html_tail();
    write2file($html,$out_file);
    prt("Html written to $out_file\n");
}

#########################################
### MAIN ###
parse_args(@ARGV);
process_in_file($in_file);
pgm_exit(0,"");
########################################

sub need_arg {
    my ($arg,@av) = @_;
    pgm_exit(1,"ERROR: [$arg] must have a following argument!\n") if (!@av);
}

sub parse_args {
    my (@av) = @_;
    my ($arg,$sarg);
    my $verb = VERB2();
    while (@av) {
        $arg = $av[0];
        if ($arg =~ /^-/) {
            $sarg = substr($arg,1);
            $sarg = substr($sarg,1) while ($sarg =~ /^-/);
            if (($sarg =~ /^h/i)||($sarg eq '?')) {
                give_help();
                pgm_exit(0,"Help exit(0)");
            } elsif ($sarg =~ /^v/) {
                if ($sarg =~ /^v.*(\d+)$/) {
                    $verbosity = $1;
                } else {
                    while ($sarg =~ /^v/) {
                        $verbosity++;
                        $sarg = substr($sarg,1);
                    }
                }
                $verb = VERB2();
                prt("Verbosity = $verbosity\n") if ($verb);
            } elsif ($sarg =~ /^l/) {
                if ($sarg =~ /^ll/) {
                    $load_log = 2;
                } else {
                    $load_log = 1;
                }
                prt("Set to load log at end. ($load_log)\n") if ($verb);
            } elsif ($sarg =~ /^o/) {
                need_arg(@av);
                shift @av;
                $sarg = $av[0];
                $out_file = $sarg;
                prt("Set out file to [$out_file].\n") if ($verb);
            } else {
                pgm_exit(1,"ERROR: Invalid argument [$arg]! Try -?\n");
            }
        } else {
            $in_file = $arg;
            prt("Set input to [$in_file]\n") if ($verb);
        }
        shift @av;
    }

    if ($debug_on) {
        prtw("WARNING: DEBUG is ON!\n");
        if (length($in_file) ==  0) {
            $in_file = $def_file;
            prt("Set DEFAULT input to [$in_file]\n");
        }
    }
    if (length($in_file) ==  0) {
        pgm_exit(1,"ERROR: No input files found in command!\n");
    }
    if (! -f $in_file) {
        pgm_exit(1,"ERROR: Unable to find in file [$in_file]! Check name, location...\n");
    }
}

sub give_help {
    prt("$pgmname: version $VERS\n");
    prt("Usage: $pgmname [options] in-file\n");
    prt("Options:\n");
    prt(" --help  (-h or -?) = This help, and exit 0.\n");
    prt(" --verb[n]     (-v) = Bump [or set] verbosity. def=$verbosity\n");
    prt(" --load        (-l) = Load LOG at end. ($outfile)\n");
    prt(" --out <file>  (-o) = Write output to this file.\n");
}

# eof - template.pl
